/***********************************************************************
File: Image.cpp
Purpose: Class for holding Image Information

Date       Author       Notes      
----------------------------------------------------------------------
11/15/05   tmorelli     Initial Version       


***********************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "Image.h"

static int debug = 1;
/*******************************************************/
Image::Image(char * _fileName)
{
  fileName = strdup(_fileName);
  data = NULL;
  InitImage();
}
/*******************************************************/
Image::Image()
{
  fileName = NULL;
  data = NULL;
}
/*******************************************************/
Image::~Image()
{
  free(fileName);
  if (data)
    free(data);
}
/*******************************************************/
void Image::InitImage()
{
  FILE * inFile = fopen(fileName, "r");

  char temp[8];
  if (debug)
    printf("Image::InitImage()\n");
  fgets(temp, 8, inFile);
  fscanf(inFile, "%d", &width);
  fscanf(inFile, "%d", &height);
  fscanf(inFile, "%d", &maxValue);

  data = (char *) malloc ((width * height * BYTES_PER_PIXEL+1) * sizeof(char));
  memset(data, 0, width * height);
  for (long  x = 0; x<=width*height*BYTES_PER_PIXEL; x++)
  {
    fscanf(inFile, "%c", &data[x]);
  }
  fclose(inFile);
}
/*******************************************************/
void Image::WriteImage(char * _fileName)
{
  FILE * outFile;
  if (_fileName)
  {
    outFile = fopen(_fileName, "w");
  }
  else if (fileName)
  {
    outFile = fopen(fileName, "w");
  }
  else
  {
    printf("Image::WriteImage() ERROR fileName is NULL\n");
    exit(0);

  }

  fprintf(outFile, "P6\n%d %d\n%d", width, height, maxValue);
  for (int x = 0; x<=width*height*BYTES_PER_PIXEL; x++)
  {
    fprintf(outFile, "%c", data[x]);
  }
  fclose(outFile);
}
/************************************************/
void Image::DrawRectangle(Point _top, Point _bottom, char _color)
{
  for (int x = _top.x; x<= _bottom.x; x++)
  {
    for (int y = _top.y; y<= _bottom.y; y++)
    {
      if ((x == _top.x || x == _bottom.x ||
          y == _top.y || y == _bottom.y))
      {
        for (int z = 0; z < BYTES_PER_PIXEL; z++)
        {
          data[RawPoint(Point(x,y))+z] = _color;
        }
      }
    }
  }
}
/************************************************/
int Image::RawPoint(Point _point)
{
  return _point.y * BYTES_PER_PIXEL * width + _point.x * BYTES_PER_PIXEL;
}
/************************************************/
void Image::SetFileName(char * _fileName)
{
  if (fileName)
    free (fileName);
  fileName = strdup(_fileName);
}
/************************************************/
void Image::FindDiffs()
{


  for (int x = 0; x<=width*height*BYTES_PER_PIXEL; x++)
  {
    if (data[x] > 0x00)
    {
      Point temp = ConvertToPoint(x);
//      printf("(%d,%d) - ",temp.x, temp.y);
    }
  }

}
/************************************************/
Point Image::ConvertToPoint(unsigned long _x)
{
  Point temp;
  temp.y = (int)floor(_x/width);
  temp.x = (int)(_x - temp.y*width);
  return temp;
}
/************************************************/
