/***********************************************************************
File: ImageProc.cpp
Purpose: Class for holding Images

Date       Author       Notes      
----------------------------------------------------------------------
11/14/05   tmorelli     Initial Version       


***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "ImageProc.h"

int debug = 1;


/************************************************/
ImageProc::ImageProc()
{
  if (debug)
    printf("ImageProc::ImageProc()\n");
  totalFiles = 0;
  totalRegions = 0;
}

/************************************************/
ImageProc::~ImageProc()
{
  if (debug)
    printf("ImageProc::~ImageProc()\n");
  for (int x = 0; x<= totalFiles; x++)
    delete inFiles[x];
  for (int x = 0; x< totalRegions; x++)
    delete regions[x];
}

/************************************************/
void ImageProc::SetBaseImage(char * _imageName)
{
  if (debug)
    printf("ImageProc::SetBaseImage(%s)\n", _imageName);
  inFiles[BASE_IMAGE] = new Image (_imageName);
//  inFiles[BASE_IMAGE]->DrawRectangle(Point(100,100), Point(200,200));
//  inFiles[BASE_IMAGE]->WriteImage("images/out.pgm");
}
/************************************************/
void ImageProc::SetNextImage(char * _imageName)
{
  if (debug)
    printf("ImageProc::SetNextImage(%s)\n", _imageName);
  inFiles[++totalFiles] = new Image (_imageName);
}
/************************************************/
Image * ImageProc::ImgSub(Image * _a, Image * _b)
{
  // Check for image sizes to be the same here
  if ((_a->width != _b->width) || (_a->height != _b->height))
  {
    printf("Cannot Subtract - Images are not the same height/width\n");
    return NULL;

  }

  Image * newImg = new Image();
  newImg->width = _a->width;
  newImg->height = _a->height;
  newImg->maxValue = _a->maxValue;

  newImg->data = (char *) malloc (BYTES_PER_PIXEL * _a->width * _a->height * sizeof(char)+1);

  for (unsigned long x = 0; x <= (BYTES_PER_PIXEL * _a->width * _a->height); x++)
  {
    if (x > 0)
      newImg->data[x] = abs(_a->data[x] - _b->data[x]);
    else
      newImg->data[x] = _a->data[x];
  }
  return newImg;
}

/************************************************/
void ImageProc::SubtractImages()
{
  inFiles[++totalFiles] = ImgSub(inFiles[0], inFiles[1]);
  if (!inFiles[totalFiles])
  {
    totalFiles --;
    return;
  }
  inFiles[totalFiles]->SetFileName("out.pgm");
  FindDiffs(inFiles[totalFiles], inFiles[totalFiles -1]);
  inFiles[totalFiles -1]->WriteImage("NewOut.pgm");
  inFiles[totalFiles]->WriteImage();
}
/************************************************/
void ImageProc::FindDiffs(Image * _in, Image * _out)
{
  for (int x = 1; x<=_in->width*_in->height*BYTES_PER_PIXEL; x++)
  {
    if (_in->data[x] > 0x00)
    {
//        _out->data[x] = 0xdf;
      Point ptTemp = ConvertToPoint(x);
      AddToRegion(ptTemp);
//      printf("(%d,%d) \n ",temp.x, temp.y);
    }
  }
  DumpRegions(_out);
}
/************************************************/
Point ImageProc::ConvertToPoint(unsigned long _x)
{
  Point temp;
  _x = floor(_x / BYTES_PER_PIXEL);
  temp.y = (int)floor(_x/inFiles[0]->width);
  temp.x = (int)(_x - temp.y*inFiles[0]->width);

  return temp;
}
/************************************************/
void ImageProc::AddToRegion(Point _point)
{
  int region = FindRegion(_point);
  if (region == -1)
  {
    regions[totalRegions] = new Region();
    regions[totalRegions]->points = (Point *) malloc 
                (inFiles[0]->width * inFiles[0]->height * sizeof(Point));
    regions[totalRegions]->points[0] = _point;
    regions[totalRegions]->totalPoints++;
    totalRegions++;

  }

}
/************************************************/
int ImageProc::FindRegion(Point _point)
{
  int region = -1; 
  for (int x = 0; x< totalRegions; x++)
  {
    for (int y = 0; y < regions[x]->totalPoints; y++)
    {
      if ( 
            (abs(regions[x]->points[y].x - _point.x) <= 2) &&
            (abs(regions[x]->points[y].y - _point.y) <= 2) 
         )
      {
        regions[x]->points[regions[x]->totalPoints] = _point;
        regions[x]->totalPoints++;
        if (regions[x]->minX > _point.x)
          regions[x]->minX = _point.x;
        if (regions[x]->minY > _point.y)
          regions[x]->minY = _point.y;
        if (regions[x]->maxX < _point.x)
          regions[x]->maxX = _point.x;
        if (regions[x]->maxY < _point.y)
          regions[x]->maxY = _point.y;
        return x;
      }
    }

  }
  //didnt find anything...
  return region;
}
/************************************************/
void ImageProc::DumpRegions(Image * _out)
{

  for (int x = 0; x< totalRegions; x++)
  {
    printf("Region: %d from (%d,%d) to (%d,%d)\n", x, regions[x]->minX,
                                     regions[x]->minY, regions[x]->maxX, 
                                     regions[x]->maxY);
    _out->DrawRectangle(Point(regions[x]->minX, regions[x]->minY), 
                        Point(regions[x]->maxX, regions[x]->maxY));
  }
}
/************************************************/
